#include "Main.h"
#include "WPrefs.h"

#include <Entry.h>
#include <File.h>
#include <fs_attr.h>
#include <FindDirectory.h>
#include <Path.h>

WPrefs::WPrefs(void)
	: sem("prefs sem")
{
	BPath path;
	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	path.Append("BWeather_settings");
	//file = new BFile(path.Path(), B_READ_WRITE | B_CREATE_FILE);
	
	file = new BFile();
	//this is only here because versions 1.0 and 1.01 had no version field in the prefs
	//so now if the prefs file exists, but it has no version field, we pretend it's version 0
	int defVersion = VERSION;
	if(file->SetTo(path.Path(), B_READ_WRITE) != B_OK)
		file->SetTo(path.Path(), B_READ_WRITE | B_CREATE_FILE);
	else
		defVersion = 0;
	
#define GetInt(value, defaultvalue) {int di=defaultvalue; _GetInt(#value, &value, &di);}
#define GetBool(value, defaultvalue) {bool db=defaultvalue; _GetBool(#value, &value, &db);}
#define GetRect(value, l, t, r, b) {BRect dr(l, t, r, b); _GetRect(#value, &value, &dr);}
#define GetString(value, defaultvalue) {char *ds=defaultvalue; _GetString(#value, value, &ds);}

	int version;
	GetInt(version, defVersion);
	if(version < PREFVERSION)
	{
		alert("Your preferences are old. They are being deleted and filled with default values.");
		delete file;
		BEntry e(path.Path());
		e.Remove();
		file = new BFile(path.Path(), B_READ_WRITE | B_CREATE_FILE);
		
		//remove locations files
		BPath path;
		find_directory(B_USER_SETTINGS_DIRECTORY, &path);
		path.Append("BWeather_locations");
		BEntry locations_file(path.Path());
		locations_file.Remove();
	}

	GetBool(opFirstOpen, true);
	GetBool(opInDeskbar, false);
	GetBool(opMetric, true);
	GetBool(opDetailed, true);
	GetInt(opUpdateSpeed, 15);
	GetRect(opWinRect, 10, 30, 110, 110);
	GetRect(opPrefRect, 200, 200, 500, 600);
	GetRect(opInfoRect, 200, 200, 600, 434);
	
	//no need to make the version numbers incompatible when all that really happened
	//was the window changed size
	if(version < 150)
		opInfoRect.bottom -= 16;
		
	GetString(opMetarServer, "http://weather.noaa.gov/cgi-bin/mgetmetar.pl?cccc=");
	GetString(opIwinServer, "http://www.weather.gov/view/prodsByState.php?state=");
	
	GetString(opLocName, "None");
	GetString(opLocCode, "----");
	GetString(opLocZone, "------");
	GetString(opLocRadar, "---");
	
	int rgb = 0;
	rgb_color c = ui_color(B_PANEL_BACKGROUND_COLOR);
	GetInt(rgb, (c.alpha<<24) | (c.red<<16) | (c.green<<8) | (c.blue<<0));
	opColor.alpha = (rgb>>24)&0xFF;
	opColor.red = (rgb>>16)&0xFF;
	opColor.green = (rgb>>8)&0xFF;
	opColor.blue = (rgb>>0)&0xFF;
	
	GetBool(opUseProxy, false);
	GetString(opProxyServer, "None");
	GetInt(opProxyPort, 0);
#undef GetInt
#undef GetBool
#undef GetRect
#undef GetString
}

WPrefs::~WPrefs(void)
{
	//make sure no one is trying to use the prefs
	sem.get();
	
	Save();
	delete file;
}

void WPrefs::Save(void)
{
#define PutInt(value) _PutInt(#value, &value)
#define PutBool(value) _PutBool(#value, &value)
#define PutRect(value) _PutRect(#value, &value)
#define PutString(value) _PutString(#value, value)
	int version = VERSION;
	PutInt(version);
	
	PutBool(opFirstOpen);
	PutBool(opInDeskbar);
	PutBool(opMetric);
	PutBool(opDetailed);
	PutInt(opUpdateSpeed);
	PutRect(opWinRect);
	PutRect(opPrefRect);
	PutRect(opInfoRect);
	PutString(opMetarServer);
	PutString(opIwinServer);
	
	PutString(opLocName);
	PutString(opLocCode);
	PutString(opLocZone);
	PutString(opLocRadar);
	
	int rgb = (opColor.alpha<<24) | (opColor.red<<16) | (opColor.green<<8) | (opColor.blue<<0);
	PutInt(rgb);
	
	PutBool(opUseProxy);
	PutString(opProxyServer);
	PutInt(opProxyPort);
#undef PutInt
#undef PutBool
#undef PutRect
#undef PutString
}

void WPrefs::_GetInt(char *name, int *value, int *defaultvalue)
{
	status_t err = file->ReadAttr(name, B_INT32_TYPE, 0, value, 4);
	if(err == B_ENTRY_NOT_FOUND)
	{
		*value = *defaultvalue;
		_PutInt(name, defaultvalue);
	}
	else if(err < 0)
	{
		alert("Unknown error reading %s:\n%s\n", name, strerror(err));
	}
}

void WPrefs::_GetBool(char *name, bool *value, bool *defaultvalue)
{
	status_t err = file->ReadAttr(name, B_BOOL_TYPE, 0, value, 1);
	if(err == B_ENTRY_NOT_FOUND)
	{
		*value = *defaultvalue;
		_PutBool(name, value);
	}else if(err < 0)
	{
		alert("Unknown error reading %s:\n%s\n", name, strerror(err));
	}
}

void WPrefs::_GetRect(char *name, BRect *value, BRect *defaultvalue)
{
	status_t err = file->ReadAttr(name, B_RECT_TYPE, 0, value, sizeof(BRect));
	if(err == B_ENTRY_NOT_FOUND)
	{
		*value = *defaultvalue;
		_PutRect(name, value);
	}
	else if(err < 0)
	{
		alert("Unknown error reading %s:\n%s\n", name, strerror(err));
	}
}

void WPrefs::_GetString(char *name, char value[], char **defaultvalue)
{
	status_t err;
	attr_info info;
	err = file->GetAttrInfo(name, &info);
	if(err == B_OK)
	{
		//*value = new char[info.size];
		file->ReadAttr(name, B_STRING_TYPE, 0, value, info.size);
	}
	else if(err == B_ENTRY_NOT_FOUND)
	{
		//*value = new char[strlen(*defaultvalue) + 1];
		strcpy(value, *defaultvalue);
		_PutString(name, value);
	}
	else
		alert("Unknown error reading %s:\n%s\n", name, strerror(err));
}



void WPrefs::_PutInt(char *name, int *value)
{
	status_t err = file->WriteAttr(name, B_INT32_TYPE, 0, value, 4);
	if(err < 0)
	{
		alert("Unknown error writing %s:\n%s\n", name, strerror(err));
	}
}

void WPrefs::_PutBool(char *name, bool *value)
{
	status_t err = file->WriteAttr(name, B_BOOL_TYPE, 0, value, 1);
	if(err < 0)
	{
		alert("Unknown error writing %s:\n%s\n", name, strerror(err));
	}
}

void WPrefs::_PutRect(char *name, BRect *value)
{
	status_t err = file->WriteAttr(name, B_RECT_TYPE, 0, value, sizeof(BRect));
	if(err < 0)
	{
		alert("Unknown error writing %s:\n%s\n", name, strerror(err));
	}
}

void WPrefs::_PutString(char *name, char value[])
{
	status_t err = file->WriteAttr(name, B_STRING_TYPE, 0, value, strlen(value) + 1);
	if(err < 0)
	{
		alert("Unknown error writing %s:\n%s\n", name, strerror(err));
	}
}
