#include "Main.h"
#include "WPrefsWindow.h"

#include "WLocations.h"
#include "WPrefs.h"
#include "WView.h"
#include "WWeather.h"
#include <Alert.h>
#include <Button.h>
#include <CheckBox.h>
#include <Font.h>
#include <ListItem.h>
#include <MessageRunner.h>
#include <OutlineListView.h>
#include <ScrollView.h>
#include <Slider.h>
#include <StringView.h>
#include <TextControl.h>

#define SHOWPROXY 0//1

WListItem::WListItem(WLocation *iloc, int level)
	: BStringItem(iloc->name, level, false), loc(iloc) {};

void WListItem::DrawItem(BView *owner, BRect itemRect, bool drawEverything)
{
	bool enabled = IsEnabled();
	SetEnabled(true);
	BStringItem::DrawItem(owner, itemRect, drawEverything);
	SetEnabled(enabled);
}

WPrefsWindow::WPrefsWindow(WPrefs *iprefs, WWeather *iweather, BMessenger *imsger)
	: BWindow(iprefs->opPrefRect,
		APP_NAME" Preferences", B_TITLED_WINDOW/*B_DOCUMENT_WINDOW*/, /*B_NOT_RESIZABLE |*/ B_NOT_ZOOMABLE),
	dontsave(false),
	prefs(iprefs),
	weather(iweather),
	msger(imsger),
	location(new WLocations)
{
	float mix, miy, max, may;
	GetSizeLimits(&mix, &max, &miy, &may);
	SetSizeLimits(250, max, 320, may);
	
	prefs->sem.get();
	
	BView *cover = new BView(Bounds(), "",
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW);
	cover->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	AddChild(cover);
	
	BRect r, locRect(Bounds());
	locRect.InsetBy(10, 10);

	//save and cancel buttons
	BButton *vOk, *vCancel;
	r.Set(0, 0, 74, 23);
	r.OffsetTo(Bounds().right - r.right - 10, Bounds().bottom - r.bottom - 10);
	vOk = new BButton(r, "", "Save", new BMessage(B_OK),
		B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM);
	vOk->MakeDefault(true);
	cover->AddChild(vOk);
	
	r.Set(0, 0, 74, 23);
	r.OffsetTo(Bounds().left + 10, Bounds().bottom - r.bottom - 10);
	vCancel = new BButton(r, "", "Cancel", new BMessage(B_CANCEL),
		B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	cover->AddChild(vCancel, vOk);

#if SHOWPROXY
	//add proxyPort textcontrol
	r = locRect;
	r.bottom = vOk->Frame().top;
	r.top = r.bottom - 15 - be_plain_font->Size();
	char buf[256];
	sprintf(buf, "%i", prefs->opProxyPort);
	vProxyPort = new BTextControl(r, "", "Proxy Server Port", buf, 0,
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_BOTTOM);
	cover->AddChild(vProxyPort, vCancel);
	
	//add proxyServer textcontrol
	r = locRect;
	r.bottom = vProxyPort->Frame().top;
	r.top = r.bottom - 15 - be_plain_font->Size();
	vProxyServer = new BTextControl(r, "", "Proxy Server Address", prefs->opProxyServer, 0,
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_BOTTOM);
	cover->AddChild(vProxyServer, vProxyPort);
	
	float d1 = vProxyPort->StringWidth("Proxy Server Port") + 5;
	float d2 = vProxyServer->StringWidth("Proxy Server Address") + 5;
	float devider;
	if(d1 > d2)
		devider = d1;
	else
		devider = d2;
	vProxyServer->SetDivider(devider);
	vProxyPort->SetDivider(devider);
	
	//add useProxy checkbox
	r = locRect;
	r.bottom = vProxyServer->Frame().top;
	r.top = r.bottom - 10 - be_plain_font->Size();
	vUseProxy = new BCheckBox(r, "", "Use Proxy Server", 0, 
		B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	vUseProxy->SetValue(prefs->opUseProxy);
	cover->AddChild(vUseProxy, vProxyServer);
#endif
	
	//add detailed checkbox
	r = locRect;
#if SHOWPROXY
	r.bottom = vUseProxy->Frame().top;
#else
	r.bottom = vOk->Frame().top;
#endif
	r.top = r.bottom - 10 - be_plain_font->Size();
	vDetailed = new BCheckBox(r, "", "Detailed Forecast", 0, 
		B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	vDetailed->SetValue(prefs->opDetailed);
#if SHOWPROXY
	cover->AddChild(vDetailed, vUseProxy);
#else
	cover->AddChild(vDetailed, vCancel);
#endif
	
	//add metric checkbox
	r = locRect;
	r.bottom = vDetailed->Frame().top;
	r.top = r.bottom - 10 - be_plain_font->Size();
	vMetric = new BCheckBox(r, "", "Metric", 0, 
		B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	vMetric->SetValue(prefs->opMetric);
	cover->AddChild(vMetric, vDetailed);
		
	//add udpate speed slider
	r = locRect;
	vUpdate = new BSlider(r, "", "Update Frequency:", 0, 0, 12, //scale of 5 minutes
		B_BLOCK_THUMB, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
	vUpdate->SetHashMarks(B_HASH_MARKS_BOTTOM);
	vUpdate->SetHashMarkCount(5);
	vUpdate->SetKeyIncrementValue(1);
	vUpdate->SetLimitLabels("0 minutes (never)", "60 minutes");
	vUpdate->SetValue(prefs->opUpdateSpeed/5);
	cover->AddChild(vUpdate);
		
	//add static caption
	r = locRect;
	r.top = vUpdate->Frame().bottom + 10;
	r.bottom = r.top + be_plain_font->Size() + 1;
	cover->AddChild(new BStringView(r, "", "Please Select your Location:",
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP));
	
	//add location list menu
	r = locRect;
	r.right -= B_V_SCROLL_BAR_WIDTH;
	r.top = vUpdate->Frame().bottom + 15 + be_plain_font->Size();
	r.bottom = vMetric->Frame().top - 10;
	vLocation = new BOutlineListView(r, "", B_SINGLE_SELECTION_LIST,
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM);
	cover->AddChild(new BScrollView("", vLocation,
		B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_FULL_UPDATE_ON_RESIZE, false, true), vMetric);
	vLocation->MakeFocus(true);
	AddLocationAt(location->head->child, 0);
	vLocation->ScrollToSelection();
	//vLocation->ScrollBy(0, vLocation->Bounds().bottom/3);
	
	AddShortcut('s', B_COMMAND_KEY, new BMessage(B_OK));
	AddShortcut('.', B_COMMAND_KEY, new BMessage(B_CANCEL));
	
	prefs->sem.rel();
}

bool del(BListItem *itm)
{
	delete itm;
	return false;
}

WPrefsWindow::~WPrefsWindow(void)
{
	delete location;
	vLocation->FullListDoForEach(del);
}

bool WPrefsWindow::QuitRequested(void)
{
	if(msger)
	{
		msger->SendMessage(WP_CLOSE);
		return false;
	}
	else
		return true;
}

void WPrefsWindow::MessageReceived(BMessage *message)
{
	BMessage msg;
	switch(message->what)
	{
		case WP_CLOSE:
			delete msger;
			msger = 0;
			SavePos();
			message->SendReply(WP_REPLY);
			PostMessage(B_QUIT_REQUESTED);
			break;
		case WP_SAVED:
			msg.what = WP_REPLY;
			msg.AddBool("saved", Saved());
			message->SendReply(&msg);
			break;
		case WP_HANDLE_DIALG:
			int32 button;
			message->FindInt32("button", &button);
			msg.what = WP_REPLY;
			msg.AddBool("rtn", HandleDialg(button));
			message->SendReply(&msg);
			break;
		case WP_ACTIVATE:
			Activate();
			break;
				
		case B_OK:
			if(!Saved())
				Save();
			msger->SendMessage(WP_CLOSE);
			break;
		case B_CANCEL:
			dontsave = true;
			msger->SendMessage(WP_CLOSE);
			break;
		case B_KEY_DOWN:
			int32 key, modifiers;
			message->FindInt32("raw_char", &key);
			message->FindInt32("modifiers", &modifiers);
			if(key == B_ESCAPE && (modifiers & B_COMMAND_KEY & B_OPTION_KEY & B_CONTROL_KEY) == 0)
				PostMessage(B_CANCEL);
			else
				BWindow::MessageReceived(message);
			break;
		default:
			BWindow::MessageReceived(message);
			break;
	}
}

int WPrefsWindow_Dialogue(void)
{
	BAlert *alert = new BAlert("BWeather", "Save BWeather Preferences?",
		"Cancel", "Don't Save", "Save",
		B_WIDTH_AS_USUAL, B_OFFSET_SPACING, B_WARNING_ALERT);
	alert->SetShortcut(0, B_ESCAPE);
	return alert->Go();
}

bool WPrefsWindow::HandleDialg(int button)
{
	if(button == 0) //cancle
		return false;
	else if(button == 1)//don't save
		return true;
	else//save
	{
		Save();
		return true;
	}
}

bool WPrefsWindow::Saved(void)
{
	if(dontsave)
		return true;
		
	bool saved = true;
	
	//determin if we need to save
	if(vUpdate->Value()*5 != prefs->opUpdateSpeed)
		saved = false;
	
	int sel = vLocation->FullListCurrentSelection();
	if(sel > 0)
	{
		WListItem *item = (WListItem*)vLocation->FullListItemAt(sel);
		if(strcmp(prefs->opLocName, item->loc->name)
			|| strcmp(prefs->opLocCode, item->loc->code)
			|| strcmp(prefs->opLocZone, item->loc->zone)
			|| strcmp(prefs->opLocRadar, item->loc->radar))
		{
			saved = false;
		}
	}
	
	if(vMetric->Value() != prefs->opMetric)
		saved = false;
	
	if(vDetailed->Value() != prefs->opDetailed)
		saved = false;

#if SHOWPROXY
	if(vUseProxy->Value() != prefs->opUseProxy)
		saved = false;
	
	if(strcmp(prefs->opProxyServer, vProxyServer->Text()))
		saved = false;
	
	if(prefs->opProxyPort != atoi(vProxyPort->Text()))
		saved = false;
#endif
	
	return saved;
}

void WPrefsWindow::Save(void)
{
	prefs->sem.get();
	
	prefs->opUpdateSpeed = vUpdate->Value()*5;

	int sel = vLocation->FullListCurrentSelection();
	if(sel > 0)
	{
		WListItem *item = (WListItem*)vLocation->FullListItemAt(sel);
		strcpy(prefs->opLocName, item->loc->name);
		strcpy(prefs->opLocCode, item->loc->code);
		strcpy(prefs->opLocZone, item->loc->zone);
		strcpy(prefs->opLocRadar, item->loc->radar);
	}
	
	prefs->opMetric = vMetric->Value();
	prefs->opDetailed = vDetailed->Value();

#if SHOWPROXY
	prefs->opUseProxy = vUseProxy->Value();
	strcpy(prefs->opProxyServer, vProxyServer->Text());
	prefs->opProxyPort = atoi(vProxyPort->Text());
#endif
	
	prefs->sem.rel();
	
	msger->SendMessage(WP_UPDATE);
}

void WPrefsWindow::SavePos(void)
{
	prefs->sem.get();
	prefs->opPrefRect = Frame();
	prefs->sem.rel();
}

void WPrefsWindow::AddLocationAt(WLocation *loc, int level)
{
	WListItem *item;
	while(loc)
	{
		item = new WListItem(loc, level);	vLocation->AddItem(item);
		
		if(loc->child)
			item->SetEnabled(false);
		
		if(!strcmp(prefs->opLocName, loc->name)
			&& !strcmp(prefs->opLocCode, loc->code)
			&& !strcmp(prefs->opLocZone, loc->zone)
			&& !strcmp(prefs->opLocRadar, loc->radar))
		{
			int i = 0;
			BListItem *list[32];
			BListItem *cur = item;
			while(cur)
			{
				list[i] = cur;
				i++;
				cur = vLocation->Superitem(cur);
			}
			while(i--)
				vLocation->Expand(list[i]);
			
			vLocation->Select(vLocation->IndexOf(item));
		}
		
		AddLocationAt(loc->child, level + 1);
		
		loc = loc->sibling;
	}
}
