#include "Main.h"
#include "WView.h"

#include "WInfoWindow.h"
#include "WPrefs.h"
#include "WPrefsWindow.h"
#include "WWeather.h"
#include <Application.h>
#include <Bitmap.h>
#include <Deskbar.h>
#include <Dragger.h>
#include <Font.h>
#include <MenuItem.h>
#include <Message.h>
#include <Messenger.h>
#include <MessageRunner.h>
#include <PopUpMenu.h>
#include <Roster.h>
#include <Window.h>

WView::WView(BRect rect, WPrefs *iprefs)
	: BView(rect, VIEW_NAME, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW),
	replicant(false),
	prefs(iprefs)
{
	Init();
	
	BRect r = Bounds();
	r.top = r.bottom - 7;
	r.left = r.right - 7;

	BDragger *dragger = new BDragger(r, this);
	AddChild(dragger);
}

WView::WView(BMessage *archive)
	: BView(archive),
	replicant(true),
	prefs(new WPrefs())
{
	Init();
}

WView::~WView(void)
{
	Quit();
}

void WView::Init(void)
{
	prefsmsger = 0;
	infomsger = 0;
	image = 0;
	imageBitch = 0;
	font = 0;
	weather = 0;
	pulseRun = 0;
	contextRun = 0;
	lastMouse.x = lastMouse.y = -1;
	dblClickValid = false;
	
	//setup the image
	MakeImage();
	
	//setup menu
	contextmenu = new BPopUpMenu("contextmenu", false, false);
//	mDeskbar = new BMenuItem("Deskbar Mode", new BMessage(WV_INDESKBAR));
	mAbout = new BMenuItem("About BWeather", new BMessage(WV_ABOUT));
	mPreferences = new BMenuItem("Preferences", new BMessage(WV_PREFERENCES));
	mUpdate = new BMenuItem("Update", new BMessage(WV_UPDATE));
	mInfo = new BMenuItem("Info", new BMessage(WV_INFO));
	mQuit = new BMenuItem("Quit", new BMessage(WV_QUIT));
	
//	contextmenu->AddItem(mDeskbar);
	contextmenu->AddSeparatorItem();
	contextmenu->AddItem(mAbout);
	contextmenu->AddItem(mPreferences);
	contextmenu->AddItem(mUpdate);
	contextmenu->AddItem(mInfo);
	contextmenu->AddSeparatorItem();
	contextmenu->AddItem(mQuit);
	
//	mDeskbar->SetMarked(prefs->opInDeskbar);
	
	SetViewColor(prefs->opColor);
}

void WView::Quit(void)
{
	ClosePrefs();
	CloseInfo();

	if(contextRun) {delete contextRun; contextRun = 0;}
	if(pulseRun) {delete pulseRun; pulseRun = 0;}
	if(weather) {delete weather; weather = 0;}
	if(prefs) {delete prefs; prefs = 0;}
	
	if(contextmenu) {delete contextmenu; contextmenu = 0;}
	if(font) {delete font; font = 0;}
	if(image) {delete image; image = 0;}
}

bool WView::QuitRequested(void)
{
	bool rtn = true;
	if(prefsmsger)
	{
		//this is a conveluted system for letting the user cancel when saving the prefs
		//first we ask the prefs if they need have been saved
		BMessage rply;
		prefsmsger->SendMessage(WP_SAVED, &rply);
		if(rply.what == WP_REPLY)
		{
			bool saved;
			rply.FindBool("saved", &saved);
			if(!saved)
			{
				//if not saved, open a dialogue and ask the user
				int32 button = WPrefsWindow_Dialogue();
				BMessage msg(WP_HANDLE_DIALG);
				msg.AddInt32("button", button);
				//tell the WPrefsWindow which button the user pressed
				prefsmsger->SendMessage(&msg, &rply);
				if(rply.what == WP_REPLY)
					rply.FindBool("rtn", &rtn);
				else
					printf("Could not ask WPrefsWindow if it should be saved.");
			}
		}
		else
			printf("Could not ask WPrefsWindow if it was saved.");
	}
	
	return rtn;
}

void WView::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case B_PULSE:
			weather->Update();
			break;
			
		case WV_HTTP_DONE:
			weather->updating = false;
			if(infomsger) infomsger->SendMessage(WI_UPDATE);
			UpdateImage();
			WriteTempFiles();
			break;
/*		case WV_INDESKBAR:
			if(QuitRequested())
			{
				bool indesk = prefs->opInDeskbar;
				prefs->sem.get();
				if(!indesk && !replicant) prefs->opWinRect = Window()->Frame();
				prefs->opInDeskbar = !prefs->opInDeskbar;
				prefs->sem.rel();
				
				Quit();
				
				//toggle being in the deskbar
				if(indesk)
				{
					RemoveFromDeskbar();
					be_roster->Launch(APP_SIGNATURE);
				}
				else
				{
					if(replicant)
						Window()->PostMessage(0x4a414841, this);
					else
						be_app->PostMessage(B_QUIT_REQUESTED);
					AddToDeskbar(new WPrefs);
				}
			}
			break;
*/		case WV_ABOUT:
		case B_ABOUT_REQUESTED:
			alert("BWeather %0.1f\n© 2000 James Marr", float(VERSION)/100);
			break;
		case WV_PREFERENCES:
			OpenPrefs();
			break;
		case WV_UPDATE:
			weather->sem.get();
			weather->BlankInfo();
			weather->updating = true;
			weather->sem.rel();
			
			if(infomsger) infomsger->SendMessage(WI_UPDATE);
			
			UpdateImage();
			weather->Update();
			break;
		case WV_INFO:
			OpenInfo();
			break;
		case WV_QUIT:
			if(QuitRequested())
			{
				if(prefs->opInDeskbar)
					RemoveFromDeskbar();
				else if(replicant)
					Window()->PostMessage(0x4a414841, this);
				else
					be_app->PostMessage(B_QUIT_REQUESTED);
			}
			break;
		case WV_CONTEXT:
			CloseContext();
			{
				BPoint point;
				message->FindPoint("where", &point);
				OpenContext(point);
			}
			break;
		
		case WP_UPDATE:
			SetPulse();
			Window()->PostMessage(WV_UPDATE, this);
			break;
		case WP_CLOSE:
			if(QuitRequested())
				ClosePrefs();
			break;
		
		case WI_CLOSE:
			CloseInfo();
			break;
		
		case 0x50535445://color. oh dear god help me
			ssize_t size;
			rgb_color *rgb;
			message->FindData((const char*)"RGBColor", (long unsigned int)B_RGB_COLOR_TYPE, (const void**)&rgb, &size);
			prefs->opColor.alpha = rgb->alpha;
			prefs->opColor.red = rgb->red;
			prefs->opColor.green = rgb->green;
			prefs->opColor.blue = rgb->blue;
			
			SetViewColor(prefs->opColor);
			UpdateImage();
			break;
			
			
		default:
			BView::MessageReceived(message);
			break;
	}
}

void WView::MouseDown(BPoint point)
{
	//if they click, they lose the contextual menu, that simple
	CloseContext();
	
	BPoint cursor;
	int32 buttons, clicks;
	MakeFocus();
	
	//get the current message from the window, and extract mouse info from it
	BMessage *message = Window()->CurrentMessage();
	message->FindInt32("buttons", &buttons);
	message->FindInt32("clicks", &clicks);
	message->FindPoint("where", &cursor);
	
	//if they right click, or hold down cnt, give that person a menu!
	if(buttons == B_SECONDARY_MOUSE_BUTTON || (buttons == B_PRIMARY_MOUSE_BUTTON && modifiers() & B_CONTROL_KEY))
		OpenContext(cursor);
	else if(buttons == B_PRIMARY_MOUSE_BUTTON)
	{
		if(clicks >= 2 && dblClickValid)
		{
			//here is how the info window is opened: when the user clicks, dblClickValid is set to true
			//if they click more than once, and dlbClickValid is still true, then open the info for them
			//however, if they move the mouse, MouseMoved() will set dblClivkValid to false
			OpenInfo();
			dblClickValid = false;
		}
		else
		{
			//reset variable for info window functionality
			dblClickValid = true;
			
			//they might be holding down the mouse button with the intent for a contextual menu
			SetContext(cursor);
		}
	}
	else
	{
		//they didn't use the propor button...
		dblClickValid = false;
	}
}

void WView::MouseUp(BPoint point)
{
	//when that mouse comes up, they are no longer trying to get a contextual menu
	CloseContext();
}

void WView::MouseMoved(BPoint point, uint32 transit, const BMessage *message)
{
	//lastMouse function is only used right here, to close the contextual menu
	//looking at transit isn't always a good indicator of whether the mouse was moved
	//this works much better
	if(lastMouse != point)
		CloseContext();
	lastMouse = point;
	
	//the mechanism for opening the info, see MouseDown above
	dblClickValid = false;
}

void WView::AttachedToWindow(void) 
{
	weather = new WWeather(prefs, this);
	weather->Update();
	SetPulse();
	//set the menu's to give their messages to us
	BMessenger messenger(this);
//	mDeskbar->SetTarget(messenger);
	mAbout->SetTarget(messenger);
	mPreferences->SetTarget(messenger);
	mUpdate->SetTarget(messenger);
	mInfo->SetTarget(messenger);
	mQuit->SetTarget(messenger);
	
	if(prefs->opInDeskbar)
		SetViewColor(Parent()->ViewColor());
	
	UpdateImage();
}

void WView::WriteTempFiles(void)
{
	//write weather files
	BFile tmpFile;
	if(tmpFile.SetTo("/tmp/bweather_weather.txt", B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE) != B_OK)
		return;
	BNode tmpNode;
	if(tmpNode.SetTo("/tmp/bweather_weather.txt") != B_OK)
		return;
	
	BMessage tmpMsg;
		
#define WRITE(name, func) \
{\
	char div[] = ": ", end[] = "\n";\
	char *val = weather->func(); \
	tmpFile.Write(name, strlen(name)); \
	tmpFile.Write(div, strlen(div)); \
	tmpFile.Write(val, strlen(val)); \
	tmpFile.Write(end, strlen(end)); \
	tmpNode.WriteAttr(name, B_STRING_TYPE, 0, val, strlen(val)+1); \
	tmpMsg.AddString(name, val); \
}
	
	WRITE("Location", GetLocStr);
	WRITE("Time", GetTimeStr);
	WRITE("Temperature", GetTempStr);
	WRITE("Dew Point", GetDewStr);
	WRITE("Sky", GetSkyStr);
	WRITE("Conditions", GetCondStr);
	WRITE("Wind", GetWindStr);
	WRITE("Visability", GetVisStr);
	WRITE("Pressure", GetPresStr);
	WRITE("Humidity", GetHumStr);

#undef WRITE
	
	int buf_size = tmpMsg.FlattenedSize();
	char *buf = new char[buf_size];
	tmpMsg.Flatten(buf, buf_size);
	tmpNode.WriteAttr("BMessage", B_RAW_TYPE, 0, buf, buf_size);
	delete[] buf;
	
	tmpFile.Unset();
	tmpNode.Unset();
	
	//write forecast files
	if(tmpFile.SetTo("/tmp/bweather_forecast.txt", B_READ_WRITE | B_CREATE_FILE | B_ERASE_FILE) != B_OK)
		return;
	
	tmpFile.Write(weather->fore, strlen(weather->fore));
	tmpFile.Unset();
}

void WView::OpenPrefs(void)
{
	if(!prefsmsger)
	{
		WPrefsWindow *prefswind = new WPrefsWindow(prefs, weather, new BMessenger(this));
		prefsmsger = new BMessenger(prefswind);
		prefswind->Show();
	}
	else
		prefsmsger->SendMessage(WP_ACTIVATE);
}

void WView::ClosePrefs(void)
{
	if(prefsmsger)
	{
		BMessage rply;
		prefsmsger->SendMessage(WP_CLOSE, &rply);
		if(rply.what != WP_REPLY)
			printf("Could not tell WPrefsWIndow to close.");
		delete prefsmsger;
		prefsmsger = 0;
	}
}

void WView::OpenInfo(void)
{
	if(!infomsger)
	{
		WInfoWindow *infowind = new WInfoWindow(prefs, weather, new BMessenger(this));
		infomsger = new BMessenger(infowind);
		infowind->Show();
	}
	else
		infomsger->SendMessage(WI_ACTIVATE);
}

void WView::CloseInfo(void)
{
	if(infomsger)
	{
		BMessage rply;
		infomsger->SendMessage(WI_CLOSE, &rply);
		if(rply.what != WI_REPLY)
			printf("Could not tell WInfoWIndow to close.");
		delete infomsger;
		infomsger = 0;
	}
}

void WView::OpenContext(BPoint point)
{
	ConvertToScreen(&point);
	BRect r(-5, -5, 5, 5);
	r = ConvertToScreen(r);
	r.OffsetTo(point);
	contextmenu->Go(point, true, true, r, true);
}

void WView::CloseContext(void)
{
	if(contextRun)
	{
		delete contextRun;
		contextRun = 0;
	}
}

void WView::SetContext(BPoint point)
{
	CloseContext();
	BMessage *msg = new BMessage(WV_CONTEXT);
	msg->AddPoint("where", point);
	contextRun = new BMessageRunner(BMessenger(this), msg, 1000000, 1);
}

void WView::SetPulse(void)
{
	if(prefs->opUpdateSpeed)
	{
		if(pulseRun)
			pulseRun->SetInterval(uint(prefs->opUpdateSpeed)*60*1000000);
		else
			pulseRun = new BMessageRunner(BMessenger(this), new BMessage(B_PULSE), uint(prefs->opUpdateSpeed)*60*1000000);
	}
	else if(pulseRun)
	{
		delete pulseRun;
		pulseRun = 0;
	}
}

void WView::Draw(BRect rect)
{
	//draw transparently
	//SetDrawingMode(B_OP_COPY);
	DrawBitmap(image, rect, rect);
	
	if(prefs->opFirstOpen)
	{
		prefs->opFirstOpen = false;
		Window()->PostMessage(WV_INFO, this);
		Window()->PostMessage(WV_PREFERENCES, this);
	}
}

void WView::MakeImage(void)
{
	//make the image, and a bitch with which to draw
	BRect rect;
	rect = Bounds();
	image = new BBitmap(rect, B_RGB32, true);
	imageBitch = new BView(rect, "imageBitch", B_FOLLOW_ALL_SIDES, 0);
	image->AddChild(imageBitch);
	
	font = new BFont();
	imageBitch->GetFont(font);
	//set the font depending on our size
	if(prefs->opInDeskbar)
	{
		font->SetSize(10);
	}
	else
	{
		font->SetSize(32);
	}
}

void WView::UpdateImage(void)
{
	image->Lock();
	
	//fill with transparent coor
	imageBitch->SetDrawingMode(B_OP_OVER);
	imageBitch->SetHighColor(ViewColor());
	imageBitch->FillRect(imageBitch->Bounds());
	
	//draw condition
	BBitmap *bmp = weather->ChooseBitmap();
	
	BRect src(bmp->Bounds()), dst(src);
	if(prefs->opInDeskbar)
		dst.OffsetTo(0, imageBitch->Bounds().bottom/2 - dst.bottom/2);
	else
		dst.OffsetTo(imageBitch->Bounds().right/2 - dst.right/2 - 15,
			imageBitch->Bounds().bottom/2 - dst.bottom/2 - 15);
	imageBitch->DrawBitmap(bmp, src, dst);
	delete bmp;
	imageBitch->Sync();
	
	if(weather->temp != VALUE_NONE && !weather->updating)
	{
		char str[255];
		sprintf(str, "%i", weather->temp);
		//if temp is only 3 characters and we're not in the deskbar, we can put the '°'
		if(weather->temp > -10 && weather->temp < 100 || !prefs->opInDeskbar)
			strcat(str, "°");
		//draw text
		rgb_color color = {0x00, 0x00, 0x00, 0xff};
		imageBitch->SetHighColor(color);
		imageBitch->SetFont(font, B_FONT_SIZE);
		BPoint p(imageBitch->Bounds().RightBottom());
		if(prefs->opInDeskbar)
		{
			p.x = p.x - imageBitch->StringWidth(str);
			p.y = p.y - 5;
		}
		else
		{
			p.x = p.x - imageBitch->StringWidth(str) - p.x/8;
			p.y = p.y - p.y/10;
		}
		imageBitch->MovePenTo(p);
		imageBitch->DrawString(str);
		imageBitch->Sync();
	}
	
	image->Unlock();
	Invalidate();
}

bool WView::IsInDeskbar(void)
{
	bool has;
	BDeskbar *deskbar = new BDeskbar();
	has = deskbar->HasItem(VIEW_NAME);
	delete deskbar;
	return has;
}

void WView::AddToDeskbar(WPrefs *prefs)
{
	//if we're running on BeOS 5 or greater we can use the entry_ref deskbar api
#if B_BEOS_VERSION >= B_BEOS_VERSION_5
	BDeskbar *deskbar = new BDeskbar();
	entry_ref ref;
	be_roster->FindApp(APP_SIGNATURE, &ref);
	status_t err = deskbar->AddItem(&ref);
	delete deskbar;
	if(err != B_OK)
		alert("Error Adding to Deskbar \"%s\"", strerror(err));
#else
	BDeskbar *deskbar = new BDeskbar();
	WView *replicant = new WView(BRect(0, 0, 16, 16), prefs);
	
	status_t err = deskbar->AddItem(replicant);
	delete replicant;
	delete deskbar;
	if(err != B_OK)
		alert("Error Adding to Deskbar \"%s\"", strerror(err));
#endif
}

void WView::RemoveFromDeskbar(void)
{
	//remove ourselves from the deskbar by name
	BDeskbar *deskbar = new BDeskbar();
	status_t err = deskbar->RemoveItem(VIEW_NAME);
	if(err != B_OK)
		alert("Error Removing fromt Deskbar \"%s\"", strerror(err));
	delete deskbar;
}

//replicant magic
BArchivable *WView::Instantiate(BMessage *data)
{
	if(!validate_instantiation(data, "WView"))
		return NULL;
	return new WView(data);
}

status_t WView::Archive(BMessage *data, bool deep) const
{
	prefs->Save();
	BView::Archive(data, deep);
	data->AddString("add_on", APP_SIGNATURE);
	data->AddString("class", "WView");
	return B_OK;
}

extern "C" _EXPORT BView *instantiate_deskbar_item();

BView *instantiate_deskbar_item()
{
	return new WView(BRect(0, 0, 16, 16), new WPrefs);
}
